<?php

namespace Drupal\rest_entity_display_menu;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\menu_link_content\Plugin\Menu\MenuLinkContent;
use Drupal\rest_entity_display\EntityDisplayManagerInterface;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Menu display manager class.
 */
class MenuDisplayManager implements MenuDisplayManagerInterface {

  use LoggerChannelTrait;

  /**
   * The entity display manager.
   *
   * @var \Drupal\rest_entity_display\EntityDisplayManagerInterface
   */
  protected EntityDisplayManagerInterface $entityDisplayManager;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The serializer.
   *
   * @var \Symfony\Component\Serializer\SerializerInterface
   */
  protected SerializerInterface $serializer;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected EntityDisplayRepositoryInterface $entityDisplayRepository;

  /**
   * Construct a new EntityDisplayManager object.
   *
   * @param \Drupal\rest_entity_display\EntityDisplayManagerInterface $entity_display_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Symfony\Component\Serializer\SerializerInterface $serializer
   *   The serializer.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct(
    EntityDisplayManagerInterface $entity_display_manager,
    ConfigFactoryInterface $config_factory,
    LanguageManagerInterface $language_manager,
    SerializerInterface $serializer,
    EntityDisplayRepositoryInterface $entity_display_repository
  ) {
    $this->entityDisplayManager = $entity_display_manager;
    $this->config = $config_factory->get('rest_entity_display_menu.settings');
    $this->languageManager = $language_manager;
    $this->serializer = $serializer;
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\rest_entity_display\Exception\EntityDisplayNotActivatedException
   * @throws \Exception
   */
  public function getNormalizedEntityDisplay(
    string $menu_id,
    array $tree,
    string $mode,
    string $context,
    string $langcode = NULL
  ): array {
    return $this->buildTree($tree, $mode, $langcode);
  }

  /**
   * Build the tree.
   *
   * @param \Drupal\Core\Menu\MenuLinkTreeElement[] $tree
   *   The tree.
   * @param string $mode
   *   The mode.
   * @param string $langcode
   *   The lang.
   *
   * @return array
   *   The normalized menu tree.
   */
  protected function buildTree(array $tree, string $mode, string $langcode = NULL) {
    $ret = [];
    foreach ($tree as $item) {
      if (!$item->link instanceof MenuLinkContent) {
        continue;
      }
      $current = $this->entityDisplayManager->getNormalizedEntityDisplay(
        $item->link->getEntity(),
        $mode,
        'view',
        $langcode
      ) + ['children' => []];
      if ($item->hasChildren) {
        $current['children'] = $this->buildTree($item->subtree ?? [], $mode, $langcode);
      }
      $ret[] = $current;
    }
    return $ret;
  }

  /**
   * {@inheritdoc}
   */
  public function getModeOptions(): array {
    return [
      'view' => $this->entityDisplayRepository->getViewModeOptions('menu_link_content')
    ];
  }

}
