<?php

namespace Drupal\rest_entity_display_menu\Plugin\rest\resource;

use Drupal\Core\Access\AccessResult;
use Drupal\rest\Annotation\RestResource;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Routing\AccessAwareRouterInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest_entity_display\EntityViewDisplayCacheManager;
use Drupal\rest_entity_display_menu\MenuDisplayManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Represents MenuTreeDisplay records as resources.
 *
 * @RestResource (
 *   id = "rest_entity_display_menu_tree_display",
 *   label = @Translation("Menu Tree display"),
 *   uri_paths = {
 *     "canonical" = "/api/menu_tree_display/{menu_id}/{mode}",
 *   }
 * )
 */
#[RestResource(
  id: 'rest_entity_display_menu_tree_display',
  label: new TranslatableMarkup('Menu Tree display'),
  uri_paths: [
    'canonical' => '/api/menu_tree_display/{menu_id}/{mode}',
  ]
)]
class MenuTreeDisplayResource extends ResourceBase {

  /**
   * The menu display manager.
   *
   * @var \Drupal\rest_entity_display_menu\MenuDisplayManagerInterface
   */
  protected MenuDisplayManagerInterface $menuDisplayManager;

  /**
   * The cache manager.
   *
   * @var \Drupal\rest_entity_display\EntityViewDisplayCacheManager
   */
  protected EntityViewDisplayCacheManager $cacheManager;

  /**
   * The menu link tree service.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected $menuTree;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected AccountProxyInterface $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    /**
     * @var \Drupal\rest_entity_display_menu\MenuDisplayManagerInterface $menu_display_manager
     */
    $menu_display_manager = $container->get('rest_entity_display_menu.menu_display_manager');
    $instance->setMenuDisplayManager($menu_display_manager);

    /**
     * @var \Drupal\rest_entity_display\EntityViewDisplayCacheManager $cache_manager
     */
    $cache_manager = $container->get('rest_entity_display.cache_manager');
    $instance->setEntityViewDisplayCacheManager($cache_manager);

    /**
     * @var \Drupal\Core\Menu\MenuLinkTreeInterface $menu_tree
     */
    $menu_tree = $container->get('menu.link_tree');
    $instance->setMenuTree($menu_tree);

    /**
     * @var \Drupal\Core\Session\AccountProxyInterface $currentUser
     */
    $currentUser = $container->get('current_user');
    $instance->setCurrentUser($currentUser);

    return $instance;
  }

  /**
   * Sets the menu display manager.
   *
   * @param \Drupal\rest_entity_display_menu\MenuDisplayManagerInterface $menu_display_manager
   *   The menu display manager.
   */
  protected function setMenuDisplayManager(MenuDisplayManagerInterface $menu_display_manager) {
    $this->menuDisplayManager = $menu_display_manager;
  }

  /**
   * Sets the cache manager.
   *
   * @param \Drupal\rest_entity_display\EntityViewDisplayCacheManager $cache_manager
   *   The cache manager.
   */
  protected function setEntityViewDisplayCacheManager(EntityViewDisplayCacheManager $cache_manager) {
    $this->cacheManager = $cache_manager;
  }

  /**
   * Sets the menu tree service.
   *
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menu_tree
   *   The menu tree service.
   */
  protected function setMenuTree(MenuLinkTreeInterface $menu_tree) {
    $this->menuTree = $menu_tree;
  }

  /**
   * Sets the current user.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   */
  protected function setCurrentUser(AccountProxyInterface $currentUser) {
    $this->currentUser = $currentUser;
  }

  /**
   * Responds to GET HTTP method.
   *
   * @param string $menu_id
   *   The menu identifier.
   * @param string $mode
   *   The form/view mode.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The incoming request.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The HTTP response.
   */
  public function get(string $menu_id, string $mode, Request $request): Response {
    try {
      $parameters = new MenuTreeParameters();
      $tree = $this->menuTree->load($menu_id, $parameters);
      $manipulators = [
        ['callable' => 'menu.default_tree_manipulators:checkAccess'],
        ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
      ];
      $tree = $this->menuTree->transform($tree, $manipulators);
      $this->cacheManager->addCacheTags(['config:system.menu' . $menu_id]);
      $data = $this->menuDisplayManager->getNormalizedEntityDisplay($menu_id, $tree, $mode, 'view');

      $response = new CacheableJsonResponse($data);
      $response->addCacheableDependency($this->cacheManager);
      $response->addCacheableDependency($request->attributes->get(AccessAwareRouterInterface::ACCESS_RESULT));

      $permission = "expose menu $menu_id view_mode $mode in rest_entity_display_menu_tree_display rest resource";
      if ($this->currentUser->hasPermission($permission)) {
        $resourceAccess = AccessResult::allowed()->cachePerPermissions();
      }
      else {
        $resourceAccess = AccessResult::forbidden("The '$permission' permission is required.")->cachePerPermissions();
      }
      $response->addCacheableDependency($resourceAccess);
    }
    catch (\Exception $e) {
      throw new NotFoundHttpException('Not found');
    }

    if (!$resourceAccess->isAllowed()) {
      throw new AccessDeniedHttpException('Not allowed');
    }
    return $response;
  }

}
