<?php

namespace Drupal\rest_entity_display;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Entity display manager class.
 */
class EntityDisplayManager implements EntityDisplayManagerInterface {

  use LoggerChannelTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The serializer.
   *
   * @var \Symfony\Component\Serializer\SerializerInterface
   */
  protected SerializerInterface $serializer;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected EntityDisplayRepositoryInterface $entityDisplayRepository;

  /**
   * Construct a new EntityDisplayManager object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Symfony\Component\Serializer\SerializerInterface $serializer
   *   The serializer.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    LanguageManagerInterface $language_manager,
    SerializerInterface $serializer,
    EntityDisplayRepositoryInterface $entity_display_repository
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $config_factory->get('rest_entity_display.settings');
    $this->languageManager = $language_manager;
    $this->serializer = $serializer;
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\rest_entity_display\Exception\EntityDisplayNotActivatedException
   * @throws \Exception
   */
  public function getNormalizedEntityDisplay(
    FieldableEntityInterface $entity,
    string $mode,
    string $context,
    ?string $langcode = NULL,
    ?RenderingContextInterface $rendering_context = NULL
  ): array {
    if ($context == 'form') {
      $entity_display = EntityFormDisplay::collectRenderDisplay($entity, $mode);
    }
    else {
      $entity_display = EntityViewDisplay::collectRenderDisplay($entity, $mode);
    }

    $normalization_context = [
      'entity' => $entity,
      'view_mode' => $mode,
      'langcode' => $langcode ?? $this->languageManager->getCurrentLanguage()->getId(),
      'rendering_context' => $rendering_context
    ];

    return $this->serializer->normalize($entity_display, 'json', $normalization_context);
  }

  /**
   * {@inheritdoc}
   */
  public function getModeOptions(string $entity_type_id): array {
    return [
      'view' => $this->entityDisplayRepository->getViewModeOptions($entity_type_id),
      'form' => $this->entityDisplayRepository->getFormModeOptions($entity_type_id),
    ];
  }

}
