<?php

namespace Drupal\rest_entity_display\Normalizer;

use Drupal\Component\Utility\SortArray;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\Display\EntityDisplayInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\rest_entity_display\EntityViewDisplayCacheManager;
use Drupal\rest_entity_display\NormalizedFormatterInterface;
use Drupal\rest_entity_display\NormalizedRenderingContextInterface;
use Drupal\rest_entity_display\RenderingContext;
use Drupal\serialization\Normalizer\ConfigEntityNormalizer;

/**
 * JSON normalizer for EntityViewDisplay config entities.
 */
class JsonEntityViewDisplayNormalizer extends ConfigEntityNormalizer {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The cache manager service.
   *
   * @var \Drupal\rest_entity_display\EntityViewDisplayCacheManager
   */
  protected EntityViewDisplayCacheManager $cacheManager;

  /**
   * The build render array.
   *
   * @var array|null
   */
  private array $build = [];

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * Construct a JsonEntityViewDisplayNormalizer object.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    EntityTypeRepositoryInterface $entity_type_repository,
    EntityFieldManagerInterface $entity_field_manager,
    LanguageManagerInterface $language_manager,
    ModuleHandlerInterface $module_handler,
    EntityViewDisplayCacheManager $cache_manager,
    ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($entity_type_manager, $entity_type_repository, $entity_field_manager);
    $this->languageManager = $language_manager;
    $this->moduleHandler = $module_handler;
    $this->cacheManager = $cache_manager;
    $this->config = $config_factory->get('rest_entity_display.settings');
  }

  /**
   * Gets the build of an entity after applying hooks.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param \Drupal\Core\Entity\Display\EntityDisplayInterface $object
   *   The object.
   * @param string $view_mode
   *   The view mode.
   *
   * @return array
   *   The build of the entity.
   */
  private function getBuild(EntityInterface $entity, EntityDisplayInterface $object, $view_mode): array {
    if (isset($this->build[$entity->getEntityTypeId()][$entity->id()])) {
      return $this->build[$entity->getEntityTypeId()][$entity->id()];
    }

    $build = [];
    $view_hook = "{$entity->getEntityTypeId()}_view";
    $this->moduleHandler->invokeAll($view_hook, [&$build, $entity, $object, $view_mode]);
    $this->moduleHandler->invokeAll('entity_view', [&$build, $entity, $object, $view_mode]);
    $this->moduleHandler->alter([$view_hook, 'entity_view'], $build, $entity, $object);

    $this->build[$entity->getEntityTypeId()][$entity->id()] = $build;

    return $build;
  }

  /**
   * {@inheritdoc}
   *
   * @var \Drupal\Core\Entity\Display\EntityViewDisplayInterface $object
   */
  public function normalize($object, $format = NULL, array $context = []): array {
    if (!isset($context['entity'])) {
      return [];
    }
    /**
     * @var \Drupal\Core\Entity\EntityInterface $entity
     */
    $entity = $context['entity'];

    /**
     * @var \Drupal\rest_entity_display\RenderingContext $parent_rendering_context
     */
    $parent_rendering_context = $context['rendering_context'];

    $view_mode = $object->getMode();

    $current_language = $context['langcode'] ?? $this->languageManager->getCurrentLanguage()->getId();
    if ($entity instanceof ContentEntityInterface && $entity->hasTranslation($current_language)) {
      $entity = $entity->getTranslation($current_language);
    }

    $cache_key = $entity->getEntityTypeId() . '_' .
      $entity->id() . '_' .
      $current_language . '_' .
      $view_mode;

    $rendering_context = (new RenderingContext())
      ->setKey($cache_key)
      ->setDepthLimit($this->config->get('render_recursive_limit'))
      ->setParentsKeys($parent_rendering_context ? $parent_rendering_context->getTreeKeys() : []);

    if (
      $rendering_context->isKeyRendered() ||
      !$rendering_context->isDepthValid()
    ) {
      return [];
    }

    $cache = &drupal_static(
      'rest_entity_display_manager_' . $cache_key
    );
    if (isset($cache)) {
      return $cache;
    }

    $this->cacheManager->addCacheableDependency($this->config);
    $this->cacheManager->addCacheableDependency($entity);
    $this->cacheManager->addCacheableDependency($object);

    $components = $object->toArray()['content'];
    uasort($components, [SortArray::class, 'sortByWeightElement']);

    $cache = [];
    foreach (array_keys($components) as $name) {
      /** @var \Drupal\Core\Field\FormatterBase $renderer */
      if ($renderer = $object->getRenderer($name)) {
        if (!$renderer instanceof NormalizedFormatterInterface) {
          continue;
        }
        if ($renderer instanceof NormalizedRenderingContextInterface) {
          $renderer->setRenderingContext($rendering_context);
        }

        /** @var \Drupal\Core\Field\FieldItemListInterface $items */
        $items = $entity->get($name);
        $items->filterEmptyItems();
        if ($items->access('view')) {
          $renderer->prepareView([$items]);
          $rest_name = $renderer->getSetting('rest_field_name') ?: $name;
          $cache[$rest_name] = $renderer->viewElements($items, $current_language);
        }
      }
      else {
        $build = $this->getBuild($entity, $object, $view_mode);
        if (array_key_exists($name, $build)) {
          $cache[$name] = $build[$name];
        }
      }

      if (!empty($this->config->get('add_entity_type_id'))) {
        $cache['entity_type'] = [['value' => $entity->getEntityTypeId()]];
      }
    }
    return $cache;
  }

  /**
   * {@inheritdoc}
   */
  public function supportsNormalization($data, $format = NULL, $context = []): bool {
    return (
      parent::supportsNormalization($data, $format) &&
      $data instanceof EntityViewDisplayInterface
    );
  }

}
