<?php

namespace Drupal\rest_entity_display\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter as CoreImageFormatter;
use Drupal\rest_entity_display\NormalizedFormatterInterface;
use Drupal\rest_entity_display\NormalizedFormatterTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\SerializerAwareTrait;

/**
 * Plugin implementation of the 'Normalized Image' formatter.
 *
 * @FieldFormatter(
 *   id = "rest_entity_display_image_formatter",
 *   label = @Translation("Normalized Image"),
 *   field_types = {
 *     "image",
 *   },
 * )
 */
#[FieldFormatter(
  id: 'rest_entity_display_image_formatter',
  label: new TranslatableMarkup('Normalized Image'),
  field_types: [
    'image',
  ]
)]
class ImageFormatter extends CoreImageFormatter implements NormalizedFormatterInterface {

  use SerializerAwareTrait;
  use NormalizedFormatterTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
  $plugin_definition
  ): self {
    $instance = parent::create(
      $container,
      $configuration,
      $plugin_id,
      $plugin_definition
    );

    /** @var \Symfony\Component\Serializer\SerializerInterface $serializer */
    $serializer = $container->get('serializer');
    $instance->setSerializer($serializer);

    /** @var \Drupal\rest_entity_display\EntityViewDisplayCacheManager $cache_manager */
    $cache_manager = $container->get('rest_entity_display.cache_manager');
    $instance->setCache($cache_manager);

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    $files = $this->getEntitiesToView($items, $langcode);

    // Early opt-out if the field is empty.
    if (empty($files)) {
      return $elements;
    }

    $url = NULL;
    $image_link_setting = $this->getSetting('image_link');
    // Check if the formatter involves a link.
    if ($image_link_setting == 'content') {
      $entity = $items->getEntity();
      if (!$entity->isNew()) {
        $url = $entity->toUrl();
      }
    }
    elseif ($image_link_setting == 'file') {
      $link_file = TRUE;
    }

    $image_style_setting = $this->getSetting('image_style');

    // Collect cache tags to be added for each item in the field.
    $base_cache_tags = [];
    if (!empty($image_style_setting)) {
      $image_style = $this->imageStyleStorage->load($image_style_setting);
      $base_cache_tags = $image_style->getCacheTags();
    }

    $itemValues = $items->getValue();
    foreach ($files as $delta => $file) {
      if (isset($link_file)) {
        $image_uri = $file->getFileUri();
        $url = $this->fileUrlGenerator->generate($image_uri);
      }
      $cache_tags = Cache::mergeTags($base_cache_tags, $file->getCacheTags());
      $this->getCache()->addCacheTags($cache_tags);

      $currentElement = [
        'url' => !empty($image_style) ? $image_style->buildUrl($file->getFileUri()) : $file->createFileUrl(),
      ];

      if (!empty($url)) {
        $currentElement['link_uri'] = $url->toString(TRUE)->getGeneratedUrl();
      }

      $attributes = array_intersect_key($itemValues[$delta], [
        'alt' => TRUE,
        'title' => TRUE,
        'width' => TRUE,
        'height' => TRUE
      ]);
      if (!empty($attributes)) {
        $currentElement['attributes'] = $attributes;
      }
      $elements[] = $currentElement;
    }

    return $elements;
  }

}
