<?php

namespace Drupal\rest_entity_display\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceFormatterBase;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\rest_entity_display\NormalizedFormatterInterface;
use Drupal\rest_entity_display\NormalizedFormatterTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\SerializerAwareTrait;

/**
 * Plugin implementation of the 'Normalized label entity reference' formatter.
 *
 * @FieldFormatter(
 *   id = "rest_entity_display_label_entity_reference_formatter",
 *   label = @Translation("Normalized label entity reference"),
 *   field_types = {
 *     "entity_reference_revisions",
 *     "entity_reference",
 *   }
 * )
 */
#[FieldFormatter(
  id: 'rest_entity_display_label_entity_reference_formatter',
  label: new TranslatableMarkup('Normalized label entity reference'),
  field_types: [
    'entity_reference_revisions',
    'entity_reference',
  ]
)]
class LabelEntityReferenceFormatter extends EntityReferenceFormatterBase implements NormalizedFormatterInterface {

  use SerializerAwareTrait;
  use NormalizedFormatterTrait;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    /** @var \Symfony\Component\Serializer\SerializerInterface $serializer */
    $serializer = $container->get('serializer');
    $instance->setSerializer($serializer);

    /** @var \Drupal\Core\Language\LanguageManagerInterface $language_manager */
    $language_manager = $container->get('language_manager');
    $instance->setLanguageManager($language_manager);

    return $instance;
  }

  /**
   * Sets the language manager.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function setLanguageManager(LanguageManagerInterface $language_manager) {
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];

    $langcode = $langcode ?? $this->languageManager->getCurrentLanguage()->getId();

    foreach ($items as $delta => $item) {
      /** @var \Drupal\Core\Entity\EntityBase $entity */
      $entity = $item->entity;
      if ($entity instanceof ContentEntityInterface && $entity->hasTranslation($langcode)) {
        $entity = $entity->getTranslation($langcode);
      }

      if ($entity) {
        $elements[$delta]['value'] = $entity->label();
      }
    }

    return $elements;
  }

}
