<?php

namespace Drupal\rest_entity_display\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter as CoreLinkFormatter;
use Drupal\rest_entity_display\NormalizedFormatterInterface;
use Drupal\rest_entity_display\NormalizedFormatterTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\SerializerAwareTrait;

/**
 * Plugin implementation of the 'link' formatter.
 *
 * @FieldFormatter(
 *   id = "rest_entity_display_link_formatter",
 *   label = @Translation("Normalized link text and URL"),
 *   field_types = {
 *     "link",
 *   }
 * )
 */
#[FieldFormatter(
  id: 'rest_entity_display_link_formatter',
  label: new TranslatableMarkup('Normalized link text and URL'),
  field_types: [
    'link',
  ]
)]
class LinkFormatter extends CoreLinkFormatter implements NormalizedFormatterInterface {
  use SerializerAwareTrait;
  use NormalizedFormatterTrait {
    defaultSettings as normalizedDefaultSettings;
    settingsForm as normalizedSettingsForm;
    settingsSummary as normalizedSettingsSummary;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    /** @var \Symfony\Component\Serializer\SerializerInterface $serializer */
    $serializer = $container->get('serializer');
    $instance->setSerializer($serializer);

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return [
      'trim_length' => '0',
    ] + static::normalizedDefaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = $this->normalizedSettingsForm($form, $form_state);
    unset($form['trim_length']);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = $this->normalizedSettingsSummary();
    array_shift($summary);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];
    $entity = $items->getEntity();

    foreach ($items as $delta => $item) {
      // By default use the full URL as the link text.
      $url = $this->buildUrl($item);

      $url_string = $url->toString(TRUE)->getGeneratedUrl();
      $link_title = $url->toString(TRUE)->getGeneratedUrl();

      // If the title field value is available, use it for the link text.
      if (!empty($item->title)) {
        // Unsanitized token replacement here because the entire link title
        // gets auto-escaped during link generation in
        // \Drupal\Core\Utility\LinkGenerator::generate().
        $link_title = \Drupal::token()->replace($item->title, [$entity->getEntityTypeId() => $entity], ['clear' => TRUE]);
      }

      $element[$delta] = [
        'title' => $link_title,
        'url' => $url_string,
        'attributes' => []
      ];

      $options = $item->options;
      $options += $url->getOptions();
      if (!empty($options['attributes'])) {
        $element[$delta]['attributes'] += $options['attributes'];
      }
      if (!empty($item->_attributes)) {
        $element[$delta]['attributes'] += $item->_attributes;
      }
    }

    return $element;
  }

}
