<?php

namespace Drupal\rest_entity_display\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\metatag\MetatagTagPluginManager;
use Drupal\rest_entity_display\NormalizedFormatterInterface;
use Drupal\rest_entity_display\NormalizedFormatterTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Normalized metatag_empty_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "rest_entity_display_metatag_empty_formatter",
 *   module = "metatag",
 *   label = @Translation("Normalized Metatag formatter"),
 *   field_types = {
 *     "metatag"
 *   }
 * )
 */
#[FieldFormatter(
  id: 'rest_entity_display_metatag_empty_formatter',
  label: new TranslatableMarkup('Normalized Metatag formatter'),
  field_types: [
    'metatag',
  ]
)]
class MetatagEmptyFormatter extends FormatterBase implements NormalizedFormatterInterface {

  use NormalizedFormatterTrait;

  /**
   * The tag plugin manager.
   *
   * @var \Drupal\metatag\MetatagTagPluginManager
   */
  protected $tagPluginManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * Set the renderer service.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function setRenderer(RendererInterface $renderer) {
    $this->renderer = $renderer;
  }

  /**
   * Set the tag plugin manager service.
   *
   * @param \Drupal\metatag\MetatagTagPluginManager $tagPluginManager
   *   The tag plugin manager service.
   */
  public function setTagPluginManager(MetatagTagPluginManager $tagPluginManager) {
    $this->tagPluginManager = $tagPluginManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    /** @var \Drupal\Core\Render\RendererInterface $renderer */
    $renderer = $container->get('renderer');
    $instance->setRenderer($renderer);

    /** @var \Drupal\metatag\MetatagTagPluginManager $tagPluginManager */
    $tagPluginManager = $container->get('plugin.manager.metatag.tag');
    $instance->setTagPluginManager($tagPluginManager);

    /** @var \Drupal\rest_entity_display\EntityViewDisplayCacheManager $cache_manager */
    $cache_manager = $container->get('rest_entity_display.cache_manager');
    $instance->setCache($cache_manager);

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    /**
     * @var \Drupal\Core\Entity\ContentEntityInterface $entity
     */
    $entity = $items->getEntity();
    $tags = $this->renderer->executeInRenderContext(new RenderContext(), static function () use ($entity) {
      return metatag_generate_entity_all_tags($entity);
    });

    $ret = [];

    $absoluteUrl = \Drupal::request()->getSchemeAndHttpHost();
    foreach ($tags as $tagPluginId => $tag) {
      $tagPluginId = preg_replace('/_\d+$/i', '', $tagPluginId);
      $tagPlugin = $this->tagPluginManager->createInstance($tagPluginId);
      $needAbsolute = $tagPlugin->requiresAbsoluteUrl() || $tagPluginId === 'image_src';
      if ($tagPluginId === 'canonical_url') {
        $tag['#attributes'][$tagPlugin->getHtmlValueAttribute()] = $entity->toUrl()->toString(TRUE)->getGeneratedUrl();
        $needAbsolute = TRUE;
      }
      elseif ($needAbsolute) {
        $tag['#attributes'][$tagPlugin->getHtmlValueAttribute()] = str_replace($absoluteUrl, '', $tag['#attributes'][$tagPlugin->getHtmlValueAttribute()]);
      }

      $ret[] = [
        'tag' => $tag['#tag'],
        'attributes' => $tag['#attributes'],
        'needAbsolute' => $needAbsolute,
        'htmlValueAttribute' => $tagPlugin->getHtmlValueAttribute(),
      ];
    }

    if ($entity instanceof ContentEntityInterface && $entity->hasLinkTemplate('canonical')) {
      // Current route represents a content entity. Build hreflang links.
      foreach ($entity->getTranslationLanguages() as $language) {
        // Skip any translation that cannot be viewed.
        $translation = $entity->getTranslation($language->getId());
        $access = $translation->access('view', NULL, TRUE);
        $this->cache->addCacheableDependency($access);
        if (!$access->isAllowed()) {
          continue;
        }
        $url = $entity->toUrl('canonical')
          ->setOption('language', $language)
          ->toString(TRUE)
          ->getGeneratedUrl();
        $ret[] = [
          'tag' => 'link',
          'attributes' => [
            'rel' => 'alternate',
            'hreflang' => $language->getId(),
            'href' => $url,
          ],
          'needAbsolute' => TRUE,
          'htmlValueAttribute' => 'href',
        ];
      }
    }



    return $ret;
  }

}
