<?php

namespace Drupal\rest_entity_display\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\media\OEmbed\Resource;
use Drupal\media\OEmbed\ResourceException;
use Drupal\media\Plugin\Field\FieldFormatter\OEmbedFormatter as CoreOEmbedFormatter;
use Drupal\rest_entity_display\NormalizedFormatterInterface;
use Drupal\rest_entity_display\NormalizedFormatterTrait;
use Drupal\rest_entity_display\NormalizedRenderingContextInterface;
use Drupal\rest_entity_display\NormalizedRenderingContextTrait;

/**
 * Plugin implementation of the 'oembed' formatter.
 *
 * @FieldFormatter(
 *   id = "rest_entity_display_oembed",
 *   label = @Translation("Normalized oEmbed content"),
 *   field_types = {
 *     "link",
 *     "string",
 *     "string_long",
 *   },
 * )
 */
#[FieldFormatter(
  id: 'rest_entity_display_oembed',
  label: new TranslatableMarkup('Normalized oEmbed content'),
  field_types: [
    'link',
    'string',
    'string_long',
  ]
)]
class OEmbedFormatter extends CoreOEmbedFormatter implements NormalizedFormatterInterface, NormalizedRenderingContextInterface {

  use NormalizedFormatterTrait;
  use NormalizedRenderingContextTrait;

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];
    $max_width = $this->getSetting('max_width');
    $max_height = $this->getSetting('max_height');

    foreach ($items as $delta => $item) {
      $main_property = $item->getFieldDefinition()->getFieldStorageDefinition()->getMainPropertyName();
      $value = $item->{$main_property};

      if (empty($value)) {
        continue;
      }

      try {
        $resource_url = $this->urlResolver->getResourceUrl($value, $max_width, $max_height);
        $resource = $this->resourceFetcher->fetchResource($resource_url);
      }
      catch (ResourceException $exception) {
        $this->logger->error("Could not retrieve the remote URL (@url).", ['@url' => $value]);
        continue;
      }

      $element[$delta] = [
        'type' => $resource->getType(),
        'provider' => $resource->getProvider()->getName(),
        'title' => $resource->getTitle(),
        'author' => [
          'name' => $resource->getAuthorName(),
          'url' => $resource->getAuthorUrl(),
        ],
        'thumbnail' => [
          'url' => $resource->getThumbnailUrl(),
          'width' => $resource->getThumbnailWidth(),
          'height' => $resource->getThumbnailHeight(),
        ],
        'width' => $max_width ?: $resource->getWidth(),
        'height' => $max_height ?: $resource->getHeight(),
        'url' => $resource->getType() === Resource::TYPE_PHOTO ? $resource->getUrl()->toString() : $value,
        'html' => $resource->getHtml(),
      ];
    }
    return $element;
  }

}
