<?php

namespace Drupal\rest_entity_display\Plugin\rest\resource;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Routing\AccessAwareRouterInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\rest\Annotation\RestResource;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest_entity_display\EntityDisplayManagerInterface;
use Drupal\rest_entity_display\EntityViewDisplayCacheManager;
use Drupal\rest_entity_display\Exception\EntityNotFieldableException;
use Drupal\rest_entity_display\Exception\EntityNotFoundException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Represents EntityDisplay records as resources.
 *
 * @RestResource (
 *   id = "rest_entity_display_entity_display",
 *   label = @Translation("Entity display"),
 *   deriver = "Drupal\rest_entity_display\Plugin\Deriver\RestEntityDisplayDeriver",
 *   uri_paths = {
 *     "canonical" = "/api/entity_{context}_display/{entity_type_id}/{identifier}/{mode}",
 *   }
 * )
 */
#[RestResource(
  id: 'rest_entity_display_entity_display',
  label: new TranslatableMarkup('Entity display'),
  deriver: 'Drupal\rest_entity_display\Plugin\Deriver\RestEntityDisplayDeriver',
  uri_paths: [
    'canonical' => '/api/entity_{context}_display/{entity_type_id}/{identifier}/{mode}',
  ]
)]
class EntityDisplayResource extends ResourceBase {

  /**
   * The entity display manager.
   *
   * @var \Drupal\rest_entity_display\EntityDisplayManagerInterface
   */
  protected EntityDisplayManagerInterface $entityDisplayManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The cache manager.
   *
   * @var \Drupal\rest_entity_display\EntityViewDisplayCacheManager
   */
  protected EntityViewDisplayCacheManager $cacheManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected AccountProxyInterface $currentUser;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    /**
     * @var \Drupal\rest_entity_display\EntityDisplayManagerInterface $entity_display_manager
     */
    $entity_display_manager = $container->get('rest_entity_display.entity_display_manager');
    $instance->setEntityDisplayManager($entity_display_manager);

    /**
     * @var \Drupal\rest_entity_display\EntityViewDisplayCacheManager $cache_manager
     */
    $cache_manager = $container->get('rest_entity_display.cache_manager');
    $instance->setEntityViewDisplayCacheManager($cache_manager);

    /**
     * @var \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
     */
    $entityTypeManager = $container->get('entity_type.manager');
    $instance->setEntityTypeManager($entityTypeManager);

    /**
     * @var \Drupal\Core\Session\AccountProxyInterface $currentUser
     */
    $currentUser = $container->get('current_user');
    $instance->setCurrentUser($currentUser);

    return $instance;
  }

  /**
   * Sets the entity display manager.
   *
   * @param \Drupal\rest_entity_display\EntityDisplayManagerInterface $entity_display_manager
   *   The entity display manager.
   */
  protected function setEntityDisplayManager(EntityDisplayManagerInterface $entity_display_manager) {
    $this->entityDisplayManager = $entity_display_manager;
  }

  /**
   * Sets the cache manager.
   *
   * @param \Drupal\rest_entity_display\EntityViewDisplayCacheManager $cache_manager
   *   The cache manager.
   */
  protected function setEntityViewDisplayCacheManager(EntityViewDisplayCacheManager $cache_manager) {
    $this->cacheManager = $cache_manager;
  }

  /**
   * Sets the entity type manager.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  protected function setEntityTypeManager(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Sets the current user.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   */
  protected function setCurrentUser(AccountProxyInterface $currentUser) {
    $this->currentUser = $currentUser;
  }

  /**
   * Responds to GET HTTP method.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param string $identifier
   *   The entity identifier.
   * @param string $mode
   *   The form/view mode.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The incoming request.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The HTTP response.
   */
  public function get(string $entity_type_id, string $identifier, string $mode, Request $request): Response {
    try {
      $definition = $this->getPluginDefinition();
      $entity = $this->getFieldableEntity($entity_type_id, $identifier);
      $data = $this->entityDisplayManager->getNormalizedEntityDisplay($entity, $mode, $definition['context']);

      $response = new CacheableJsonResponse($data);
      $response->addCacheableDependency($this->cacheManager);
      $response->addCacheableDependency($request->attributes->get(AccessAwareRouterInterface::ACCESS_RESULT));

      $entityAccess = $entity->access($definition['context'] === 'form' ? 'update' : 'view', NULL, TRUE);
      $response->addCacheableDependency($entityAccess);

      $permission = "expose $entity_type_id {$definition['context']}_mode $mode in rest_entity_display rest resource";
      if ($this->currentUser->hasPermission($permission)) {
        $resourceAccess = AccessResult::allowed()->cachePerPermissions();
      }
      else {
        $resourceAccess = AccessResult::forbidden("The '$permission' permission is required.")->cachePerPermissions();
      }
      $response->addCacheableDependency($resourceAccess);
    }
    catch (\Exception $e) {
      throw new NotFoundHttpException('Not found');
    }

    if (!$entityAccess->isAllowed() || !$resourceAccess->isAllowed()) {
      throw new AccessDeniedHttpException('Not allowed');
    }
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldableEntity(string $entity_type_id, string $id): FieldableEntityInterface {
    $entity = $this->entityTypeManager->getStorage($entity_type_id)->load($id);
    if (!$entity) {
      throw new EntityNotFoundException();
    }
    if (!($entity instanceof FieldableEntityInterface)) {
      throw new EntityNotFieldableException();
    }

    return $entity;
  }

}
