<?php

namespace Drupal\rest_entity_display;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\BundlePermissionHandlerTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a class containing permission callbacks.
 */
class RestEntityDisplayPermissions implements ContainerInjectionInterface {

  use StringTranslationTrait;
  use BundlePermissionHandlerTrait;

  /**
   * Constructs a RestEntityDisplayPermissions instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\rest_entity_display\EntityDisplayManagerInterface $entityDisplayManager
   *   Entity display manager.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityDisplayManagerInterface $entityDisplayManager
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('rest_entity_display.entity_display_manager')
    );
  }

  /**
   * Returns an array of rest_entity_display type permissions.
   *
   * @return array
   *   The rest_entity_display type permissions.
   *
   * @see \Drupal\user\PermissionHandlerInterface::getPermissions()
   */
  public function permissions() {
    $perms = [];
    $entityTypes = $this->entityTypeManager->getDefinitions();
    foreach ($entityTypes as $entityType) {
      if (
        !$entityType->hasViewBuilderClass() ||
        !$entityType->entityClassImplements(FieldableEntityInterface::class)
      ) {
        continue;
      }
      foreach ($this->entityDisplayManager->getModeOptions($entityType->id()) as $mode_type => $mode) {
        foreach ($mode as $mode_id => $current_mode) {
          $perms["expose {$entityType->id()} {$mode_type}_mode $mode_id in rest_entity_display rest resource"] = [
            'title' => $this->t('Expose %type_name %mode_type mode "%mode" in Rest entity display resource.', [
              '%type_name' => $entityType->getLabel(),
              '%mode_type' => $mode_type,
              '%mode' => (string) $current_mode,
            ])
          ];
        }
      }
    }
    return $perms;
  }

}
